# ArchProbe

ArchProbe is a profiling tool to demythify mobile GPU architectures with great details. The mechanism of ArchProbe is introduced in our technical paper which is still under review.

![Adreno & Mali Architecture Overview](overview.png)  
*Architecture details collected with ArchProbe, presented in our technical paper.*

## How to Use

In a clone of ArchProbe code repository, the following commands build ArchProbe for most mobile devices with a 64-bit ARMv8 architecture.

```powershell
git submodule update --init --recursive
mkdir build-android-aarch64 && cd build-android-aarch64
cmake -DCMAKE_TOOLCHAIN_FILE="$ANDROID_NDK/build/cmake/android.toolchain.cmake" -DANDROID_ABI="arm64-v8a" -DANDROID_PLATFORM=android-28 -G "Ninja" ..
cmake --build . -t ArchProbe
```

To run ArchProbe in command line via `adb shell`, you need to copy the executables to `/data/local/tmp`.

If you are using Windows, the PowerShell scripts in `scripts` can be convenient too:

```powershell
scripts/Run-Android.ps1 [-Verbose]
```

### Prebuilt Binaries

Prebuilt binaries will be available [here](https://github.com/PENGUINLIONG/graphi-t/releases).

## How to Interpret Outputs

A GPU hardware has many traits like GFLOPS and cache size. ArchProbe implements a bag of tricks to expose these traits and each implementation is called an *aspect*. Each aspect has its own configurations in `ArchProbe.json`, reports in `ArchProbeReport.json`, and data table of every run of probing kernels in `[ASPECT_NAME].csv`. Currently ArchProbe implements the following aspects:

- `WarpSizeMethod{A|B}` Two methods to detect the warp size of a GPU core;
- `GFLOPS` Peak computational throughput of the device;
- `RegCount` Number of registers available to a thread and whether the register file is shared among warps;
- `BufferVecWidth` Optimal vector width to read the most data in a single memory access;
- `{Image|Buffer}CachelineSize` Top level cacheline size of image/buffer;
- `{Image|Buffer}Bandwidth` Peak read-only bandwidth of image/buffer;
- `{Image|Buffer}CacheHierarchyPChase` Size of each level of cache of image/buffer by the P-chase method.

If the `-v` flag is given, ArchProbe prints extra human-readable logs to `stdout` which is also a good source of information.

Experiment data gathered from Google Pixel 4 can be found [here](examples/adreno640/Google_Pixel_4).

## Contributing

This project welcomes contributions and suggestions.  Most contributions require you to agree to a
Contributor License Agreement (CLA) declaring that you have the right to, and actually do, grant us
the rights to use your contribution. For details, visit https://cla.opensource.microsoft.com.

When you submit a pull request, a CLA bot will automatically determine whether you need to provide
a CLA and decorate the PR appropriately (e.g., status check, comment). Simply follow the instructions
provided by the bot. You will only need to do this once across all repos using our CLA.

This project has adopted the [Microsoft Open Source Code of Conduct](https://opensource.microsoft.com/codeofconduct/).
For more information see the [Code of Conduct FAQ](https://opensource.microsoft.com/codeofconduct/faq/) or
contact [opencode@microsoft.com](mailto:opencode@microsoft.com) with any additional questions or comments.

## Trademarks

This project may contain trademarks or logos for projects, products, or services. Authorized use of Microsoft 
trademarks or logos is subject to and must follow 
[Microsoft's Trademark & Brand Guidelines](https://www.microsoft.com/en-us/legal/intellectualproperty/trademarks/usage/general).
Use of Microsoft trademarks or logos in modified versions of this project must not cause confusion or imply Microsoft sponsorship.
Any use of third-party trademarks or logos are subject to those third-party's policies.
